@@ -4,7 +4,7 @@
 ## Department of Probabilities & Statistics
 ## Faculty of Mathematics
 ## University of Science and Technology Houari Boumediene
-## BP 32 El-Alia, U.S.T.H.B, Algeris
+## BP 32 El-Alia, U.S.T.H.B, Algiers
 ## Algeria
 
 ## This program is free software; you can redistribute it and/or modify
@@ -25,17 +25,12 @@
 
 #####
 ##### SMilstein1D
-
-.SMilstein1D <- function(N =100,M=1,x0=2,t0=0,T=1,Dt,drift,diffusion,
+ 
+.SMilstein1D <- function(N =100,M=1,x0=1,t0=0,T=1,Dt,drift,diffusion,
                           type=c("ito","str"),...)
                        {
-    if (missing(Dt)) {
-        t <- seq(t0, T, length = N + 1)
-    } else {
-        t <- c(t0, t0 + cumsum(rep(Dt, N)))
-        T <- t[N + 1]
-    } 					   
-    if (missing(type)) type <- "ito"   					   
+    DSx  <- D(diffusion,"x")  
+    DSxx <- D(DSx,"x")					     					   
     if (type=="ito"){
     A    <- function(t,x)  eval(drift)
     Ax   <- function(t,x)  eval(D(drift,"x"))
@@ -46,53 +41,36 @@
     Axx  <- function(t,x)  eval(D(D(drift,"x"),"x")) - 0.5 * ( eval(D(D(diffusion,"x"),"x")) * eval(D(diffusion,"x"))+ eval(D(diffusion,"x")) * eval(D(D(diffusion,"x"),"x"))+
                            eval(D(diffusion,"x")) * eval(D(D(diffusion,"x"),"x")) + eval(diffusion) * eval(D(D(D(diffusion,"x"),"x"),"x")) )
                   }
-    DSx  <- D(diffusion,"x")  
-    DSxx <- D(DSx,"x")
     S    <- function(t,x)  eval(diffusion)
     Sx   <- function(t,x)  eval(DSx)
     Sxx  <- function(t,x)  eval(DSxx)
-    smilstein1d <- function()
-           {
-    w = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    dw   <- diff(w)
-    X    <- numeric()
-    X[1] <- x0 
-    for (i in 2:(N+1)){
-       X[i] = X[i-1] + A(t[i-1],X[i-1])*Dt + S(t[i-1],X[i-1])*dw[i-1] +0.5 *S(t[i-1],X[i-1]) *
-              Sx(t[i-1],X[i-1])*(dw[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *A(t[i-1],X[i-1])*Sx(t[i-1],X[i-1]) + 
-              0.5 * Ax(t[i-1],X[i-1])*S(t[i-1],X[i-1])+ 0.25 *(S(t[i -1] ,X[i -1])^2) *Sxx(t[i -1] ,X[i -1]))*
-              dw[i -1]+ (Dt^2) * (0.5*A(t[i -1],X[i -1])*Ax(t[i-1],X[i-1])+ 0.25 *Axx(t[i-1],X[i-1])*(S(t[i-1],X[i-1])^2))
-                      }
-    X
-         }
-    res <- data.frame(replicate(M,smilstein1d()))
-    names(res) <- paste("X",1:M,sep="")
-    X <- ts(res, start = t0, deltat = Dt)
-    structure(list(X=X, drift=drift[[1]], diffusion=diffusion[[1]],type=type, x0=x0, N=N, M = M, 
-                   Dt=Dt,t0=t0,T=T),class="snssde1d")
-}           
+    x0 <- rep(x0,M)[1:M]	
+    X <- .Call("SMilstein1d", x0, t0, Dt, as.integer(N), as.integer(M), A, Ax, Axx, S, Sx, Sxx, .GlobalEnv, PACKAGE="Sim.DiffProc")
+    name <- "X"
+    name <- if(M > 1) paste("X",1:M,sep="")
+    X <- ts(X, start = t0, deltat = Dt, names=name)
+    return(list(X=X))
+}   
+      
+        
 
 #####
 ##### SMilstein2D
 
-.SMilstein2D <- function(N =100,x0=2,y0=1,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
+.SMilstein2D <- function(N =100,M=1,x0=2,y0=1,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
                           type=c("ito","str"),...)
                        {
-    if (missing(Dt)) {
-        t <- seq(t0, T, length = N + 1)
-    } else {
-        t <- c(t0, t0 + cumsum(rep(Dt, N)))
-        T <- t[N + 1]
-    } 					   
-    if (missing(type)) type <- "ito" 					   
+    DSx  <- D(diffx,"x")  
+    DSxx <- D(DSx,"x")
+    DSy  <- D(diffy,"y")  
+    DSyy <- D(DSy,"y")					   
     if (type=="ito"){
     Ax    <- function(t,x,y)  eval(driftx)
     dAx   <- function(t,x,y)  eval(D(driftx,"x"))
     dAxx  <- function(t,x,y)  eval(D(D(driftx,"x"),"x"))
     Ay    <- function(t,x,y)  eval(drifty)
     dAy   <- function(t,x,y)  eval(D(drifty,"y"))
-    dAyy  <- function(t,x,y)  eval(D(D(drifty,"y"),"y"))
-    }else{
+    dAyy  <- function(t,x,y)  eval(D(D(drifty,"y"),"y"))}else{
     Ax    <- function(t,x,y)  eval(driftx) - 0.5 * eval(diffx) * eval(D(diffx,"x"))
     dAx   <- function(t,x,y)  eval(D(driftx,"x")) - 0.5 * (eval(D(diffx,"x")) * eval(D(diffx,"x"))+ eval(diffx) * eval(D(D(diffx,"x"),"x")))
     dAxx  <- function(t,x,y)  eval(D(D(driftx,"x"),"x")) - 0.5 * ( eval(D(D(diffx,"x"),"x")) * eval(D(diffx,"x"))+ eval(D(diffx,"x")) * eval(D(D(diffx,"x"),"x"))+
@@ -102,54 +80,28 @@
     dAyy  <- function(t,x,y)  eval(D(D(drifty,"y"),"y")) - 0.5 * ( eval(D(D(diffy,"y"),"y")) * eval(D(diffy,"y"))+ eval(D(diffy,"y")) * eval(D(D(diffy,"y"),"y"))+
                               eval(D(diffy,"y")) * eval(D(D(diffy,"y"),"y")) + eval(diffy) * eval(D(D(D(diffy,"y"),"y"),"y")) )
                   }
-    DSx  <- D(diffx,"x")  
-    DSxx <- D(DSx,"x")
     Sx    <- function(t,x,y)  eval(diffx)
     dSx   <- function(t,x,y)  eval(DSx)
     dSxx  <- function(t,x,y)  eval(DSxx)
-    DSy  <- D(diffy,"y")  
-    DSyy <- D(DSy,"y")
     Sy    <- function(t,x,y)  eval(diffy)
     dSy   <- function(t,x,y)  eval(DSy)
     dSyy  <- function(t,x,y)  eval(DSyy)
-    wx = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    wy = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    dwx   <- diff(wx)
-    dwy   <- diff(wy)
-    X = Y <- numeric()
-    X[1] <- x0 
-    Y[1] <- y0
-    for (i in 2:(N+1)){
-       X[i] = X[i-1] + Ax(t[i-1],X[i-1],Y[i-1])*Dt + Sx(t[i-1],X[i-1],Y[i-1])*dwx[i-1] +0.5 *Sx(t[i-1],X[i-1],Y[i-1]) *
-              dSx(t[i-1],X[i-1],Y[i-1])*(dwx[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *Ax(t[i-1],X[i-1],Y[i-1])*dSx(t[i-1],X[i-1],Y[i-1]) + 
-              0.5 * dAx(t[i-1],X[i-1],Y[i-1])*Sx(t[i-1],X[i-1],Y[i-1])+ 0.25 *(Sx(t[i -1] ,X[i -1],Y[i-1])^2) *dSxx(t[i -1] ,X[i -1],Y[i-1]))*
-              dwx[i -1]+ (Dt^2) * (0.5*Ax(t[i -1],X[i -1],Y[i-1])*dAx(t[i-1],X[i-1],Y[i-1])+ 0.25 *dAxx(t[i-1],X[i-1],Y[i-1])*(Sx(t[i-1],X[i-1],Y[i-1])^2))
-       Y[i] = Y[i-1] + Ay(t[i-1],X[i-1],Y[i-1])*Dt + Sy(t[i-1],X[i-1],Y[i-1])*dwy[i-1] +0.5 *Sy(t[i-1],X[i-1],Y[i-1]) *
-              dSy(t[i-1],X[i-1],Y[i-1])*(dwy[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *Ay(t[i-1],X[i-1],Y[i-1])*dSy(t[i-1],X[i-1],Y[i-1]) + 
-              0.5 * dAy(t[i-1],X[i-1],Y[i-1])*Sy(t[i-1],X[i-1],Y[i-1])+ 0.25 *(Sy(t[i -1] ,X[i -1],Y[i-1])^2) *dSyy(t[i -1] ,X[i -1],Y[i-1]))*
-              dwy[i -1]+ (Dt^2) * (0.5*Ay(t[i -1],X[i -1],Y[i-1])*dAy(t[i-1],X[i-1],Y[i-1])+ 0.25 *dAyy(t[i-1],X[i-1],Y[i-1])*(Sy(t[i-1],X[i-1],Y[i-1])^2))
-                      }
-    res <- data.frame(X,Y)
-    names(res) <- paste(c("X","Y"),sep="")
-    X <- ts(res, start = t0, deltat = Dt)
-    structure(list(X=X, driftx=driftx[[1]], diffx=diffx[[1]],drifty=drifty[[1]], diffy=diffy[[1]],
-                  type=type, x0=x0,y0=y0, N=N, Dt=Dt,t0=t0,T=T),class="snssde2d")
-}  
-
+    x0 <- rep(x0,M)[1:M]
+    y0 <- rep(y0,M)[1:M]	
+    Val <- .Call("SMilstein2d", x0, y0, t0, Dt, as.integer(N), as.integer(M), Ax, dAx,dAxx, Ay, dAy,dAyy, Sx, dSx, dSxx, Sy, dSy, dSyy, .GlobalEnv, PACKAGE="Sim.DiffProc")
+    name <- c("X","Y")
+    name <- if(M > 1) c(paste(name[1],1:M,sep=""),paste(name[2],1:M,sep=""))
+    X <- ts(Val[,1:M], start = t0, deltat = Dt, names=name[1:M])
+    Y <- ts(Val[,(M+1):(2*M)], start = t0, deltat = Dt, names=name[(M+1):(2*M)])
+    return(list(X=X,Y=Y))
+} 
 
 #####
 ##### SMilstein3D
 
-.SMilstein3D <- function(N =100,x0=2,y0=1,z0=1,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
+.SMilstein3D <- function(N =100,M=1,x0=2,y0=1,z0=1,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
                      driftz,diffz,type=c("ito","str"),...)
                        {
-    if (missing(Dt)) {
-        t <- seq(t0, T, length = N + 1)
-    } else {
-        t <- c(t0, t0 + cumsum(rep(Dt, N)))
-        T <- t[N + 1]
-    } 					   
-    if (missing(type)) type <- "ito" 					   
     if (type=="ito"){
     Ax    <- function(t,x,y,z)  eval(driftx)
     dAx   <- function(t,x,y,z)  eval(D(driftx,"x"))
@@ -188,36 +140,18 @@
     Sz    <- function(t,x,y,z)  eval(diffz)
     dSz   <- function(t,x,y,z)  eval(DSz)
     dSzz  <- function(t,x,y,z)  eval(DSzz)
-    wx = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    wy = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    wz = c(0,cumsum(rnorm(N,mean=0,sd=sqrt(Dt))))
-    dwx   <- diff(wx)
-    dwy   <- diff(wy)
-    dwz   <- diff(wz)
-    X = Y = Z <- numeric()
-    X[1] <- x0 
-    Y[1] <- y0
-    Z[1] <- z0
-    for (i in 2:(N+1)){
-       X[i] = X[i-1] + Ax(t[i-1],X[i-1],Y[i-1],Z[i-1])*Dt + Sx(t[i-1],X[i-1],Y[i-1],Z[i-1])*dwx[i-1] +0.5 *Sx(t[i-1],X[i-1],Y[i-1],Z[i-1]) *
-              dSx(t[i-1],X[i-1],Y[i-1],Z[i-1])*(dwx[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *Ax(t[i-1],X[i-1],Y[i-1],Z[i-1])*dSx(t[i-1],X[i-1],Y[i-1],Z[i-1]) + 
-              0.5 * dAx(t[i-1],X[i-1],Y[i-1],Z[i-1])*Sx(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *(Sx(t[i -1] ,X[i -1],Y[i-1],Z[i-1])^2) *dSxx(t[i -1] ,X[i -1],Y[i-1],Z[i-1]))*
-              dwx[i -1]+ (Dt^2) * (0.5*Ax(t[i -1],X[i -1],Y[i-1],Z[i-1])*dAx(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *dAxx(t[i-1],X[i-1],Y[i-1],Z[i-1])*(Sx(t[i-1],X[i-1],Y[i-1],Z[i-1])^2))
-       Y[i] = Y[i-1] + Ay(t[i-1],X[i-1],Y[i-1],Z[i-1])*Dt + Sy(t[i-1],X[i-1],Y[i-1],Z[i-1])*dwy[i-1] +0.5 *Sy(t[i-1],X[i-1],Y[i-1],Z[i-1]) *
-              dSy(t[i-1],X[i-1],Y[i-1],Z[i-1])*(dwy[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *Ay(t[i-1],X[i-1],Y[i-1],Z[i-1])*dSy(t[i-1],X[i-1],Y[i-1],Z[i-1]) + 
-              0.5 * dAy(t[i-1],X[i-1],Y[i-1],Z[i-1])*Sy(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *(Sy(t[i -1] ,X[i -1],Y[i-1],Z[i-1])^2) *dSyy(t[i -1] ,X[i -1],Y[i-1],Z[i-1]))*
-              dwy[i -1]+ (Dt^2) * (0.5*Ay(t[i -1],X[i -1],Y[i-1],Z[i-1])*dAy(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *dAyy(t[i-1],X[i-1],Y[i-1],Z[i-1])*(Sy(t[i-1],X[i-1],Y[i-1],Z[i-1])^2))
-       Z[i] = Z[i-1] + Az(t[i-1],X[i-1],Y[i-1],Z[i-1])*Dt + Sz(t[i-1],X[i-1],Y[i-1],Z[i-1])*dwz[i-1] +0.5 *Sz(t[i-1],X[i-1],Y[i-1],Z[i-1]) *
-              dSz(t[i-1],X[i-1],Y[i-1],Z[i-1])*(dwz[i-1]^2-Dt)+ Dt^(3 /2)*(0.5 *Az(t[i-1],X[i-1],Y[i-1],Z[i-1])*dSz(t[i-1],X[i-1],Y[i-1],Z[i-1]) + 
-              0.5 * dAz(t[i-1],X[i-1],Y[i-1],Z[i-1])*Sz(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *(Sz(t[i -1] ,X[i -1],Y[i-1],Z[i-1])^2) *dSzz(t[i -1] ,X[i -1],Y[i-1],Z[i-1]))*
-              dwz[i -1]+ (Dt^2) * (0.5*Az(t[i -1],X[i -1],Y[i-1],Z[i-1])*dAz(t[i-1],X[i-1],Y[i-1],Z[i-1])+ 0.25 *dAzz(t[i-1],X[i-1],Y[i-1],Z[i-1])*(Sz(t[i-1],X[i-1],Y[i-1],Z[i-1])^2))
-                      }
-    res <- data.frame(X,Y,Z)
-    names(res) <- paste(c("X","Y","Z"),sep="")
-    X <- ts(res, start = t0, deltat = Dt)
-    structure(list(X=X, driftx=driftx[[1]], diffx=diffx[[1]],drifty=drifty[[1]], diffy=diffy[[1]],
-                  driftz=driftz[[1]], diffz=diffz[[1]],type=type, x0=x0,y0=y0,z0=z0, N=N, Dt=Dt,t0=t0,T=T),class="snssde3d")
-} 
+    x0 <- rep(x0,M)[1:M]
+    y0 <- rep(y0,M)[1:M]
+    z0 <- rep(z0,M)[1:M]
+    Val <- .Call("SMilstein3d", x0, y0, z0, t0, Dt, as.integer(N), as.integer(M), Ax,dAx, dAxx, Ay,dAy, dAyy, Az,dAz, dAzz,
+                 Sx, dSx, dSxx, Sy, dSy, dSyy, Sz, dSz, dSzz, .GlobalEnv, PACKAGE="Sim.DiffProc")
+    name <- c("X","Y","Z")
+    name <- if(M > 1) c(paste(name[1],1:M,sep=""),paste(name[2],1:M,sep=""),paste(name[3],1:M,sep=""))
+    X <- ts(Val[,1:M], start = t0, deltat = Dt, names=name[1:M])
+    Y <- ts(Val[,(M+1):(2*M)], start = t0, deltat = Dt, names=name[(M+1):(2*M)])
+    Z <- ts(Val[,(2*M+1):(3*M)], start = t0, deltat = Dt, names=name[(2*M+1):(3*M)])
+    return(list(X=X,Y=Y,Z=Z))
+}
 
 
 
